@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2020-2024, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the LICENSE.md file
 // distributed with the sources of this project regarding your rights to use or distribute this
 // software.
@@ -7,15 +7,16 @@ package integrity
 
 import (
 	"bytes"
+	"context"
 	"crypto"
 	"errors"
 	"os"
 	"path/filepath"
-	"reflect"
+	"slices"
 	"testing"
 
 	"github.com/ProtonMail/go-crypto/openpgp"
-	"github.com/sebdah/goldie/v2"
+	"github.com/ProtonMail/go-crypto/openpgp/packet"
 	"github.com/sylabs/sif/v2/pkg/sif"
 )
 
@@ -68,7 +69,6 @@ func TestOptSignGroupObjects(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			gs := groupSigner{f: twoGroupImage, id: tt.groupID}
 
@@ -82,7 +82,7 @@ func TestOptSignGroupObjects(t *testing.T) {
 				for _, od := range gs.ods {
 					got = append(got, od.ID())
 				}
-				if want := tt.ids; !reflect.DeepEqual(got, want) {
+				if want := tt.ids; !slices.Equal(got, want) {
 					t.Errorf("got objects %v, want %v", got, want)
 				}
 			}
@@ -171,9 +171,9 @@ func TestNewGroupSigner(t *testing.T) {
 			name:        "OptSignGroupMetadataHash",
 			fi:          twoGroupImage,
 			groupID:     1,
-			opts:        []groupSignerOpt{optSignGroupMetadataHash(crypto.SHA1)},
+			opts:        []groupSignerOpt{optSignGroupMetadataHash(crypto.SHA384)},
 			wantObjects: []uint32{1, 2},
-			wantMDHash:  crypto.SHA1,
+			wantMDHash:  crypto.SHA384,
 		},
 		{
 			name:    "OptSignGroupMetadataHash",
@@ -195,9 +195,8 @@ func TestNewGroupSigner(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
-			en := newClearsignEncoder(getTestEntity(t), fixedTime)
+			en := newClearsignEncoder(getTestEntity(t), &packet.Config{Time: fixedTime})
 
 			s, err := newGroupSigner(en, tt.fi, tt.groupID, tt.opts...)
 			if got, want := err, tt.wantErr; !errors.Is(got, want) {
@@ -221,7 +220,7 @@ func TestNewGroupSigner(t *testing.T) {
 				for _, od := range s.ods {
 					got = append(got, od.ID())
 				}
-				if want := tt.wantObjects; !reflect.DeepEqual(got, want) {
+				if want := tt.wantObjects; !slices.Equal(got, want) {
 					t.Errorf("got objects %v, want %v", got, want)
 				}
 
@@ -256,12 +255,12 @@ func TestGroupSigner_Sign(t *testing.T) {
 	}
 
 	e := getTestEntity(t)
-	clearsign := newClearsignEncoder(e, fixedTime)
+	clearsign := newClearsignEncoder(e, &packet.Config{Time: fixedTime})
 
 	encrypted := getTestEntity(t)
 	encrypted.PrivateKey.Encrypted = true
 
-	clearsignEncrypted := newClearsignEncoder(encrypted, fixedTime)
+	clearsignEncrypted := newClearsignEncoder(encrypted, &packet.Config{Time: fixedTime})
 
 	tests := []struct {
 		name    string
@@ -287,7 +286,7 @@ func TestGroupSigner_Sign(t *testing.T) {
 				f:      twoGroups,
 				id:     1,
 				ods:    []sif.Descriptor{d1},
-				mdHash: crypto.SHA1,
+				mdHash: crypto.SHA256,
 				fp:     encrypted.PrimaryKey.Fingerprint,
 			},
 			wantErr: true,
@@ -339,9 +338,8 @@ func TestGroupSigner_Sign(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
-			di, err := tt.gs.sign()
+			di, err := tt.gs.sign(context.Background())
 			if (err != nil) != tt.wantErr {
 				t.Fatalf("got error %v, want %v", err, tt.wantErr)
 			}
@@ -362,12 +360,33 @@ func TestGroupSigner_Sign(t *testing.T) {
 					}
 				})
 
-				if err := fi.UnloadContainer(); err != nil {
-					t.Error(err)
+				d, err := fi.GetDescriptor(sif.WithDataType(sif.DataSignature))
+				if err != nil {
+					t.Fatal(err)
 				}
 
-				g := goldie.New(t, goldie.WithTestNameForDir(true))
-				g.Assert(t, tt.name, buf.Bytes())
+				if got, want := d.GroupID(), uint32(0); got != want {
+					t.Errorf("got group ID %v, want %v", got, want)
+				}
+
+				id, isGroup := d.LinkedID()
+				if got, want := id, tt.gs.id; got != want {
+					t.Errorf("got linked ID %v, want %v", got, want)
+				}
+				if !isGroup {
+					t.Error("got linked object, want linked group")
+				}
+
+				ht, fp, err := d.SignatureMetadata()
+				if err != nil {
+					t.Fatal(err)
+				}
+				if got, want := ht, crypto.SHA256; got != want {
+					t.Errorf("got hash %v, want %v", got, want)
+				}
+				if got, want := fp, tt.gs.fp; !bytes.Equal(got, want) {
+					t.Errorf("got fingerprint %v, want %v", got, want)
+				}
 			}
 		})
 	}
@@ -431,6 +450,11 @@ func TestNewSigner(t *testing.T) {
 			},
 			wantErr: sif.ErrNoObjects,
 		},
+		{
+			name:    "NoKeyMaterial",
+			fi:      oneGroupImage,
+			wantErr: ErrNoKeyMaterial,
+		},
 		{
 			name: "InvalidObjectID",
 			fi:   oneGroupImage,
@@ -513,7 +537,6 @@ func TestNewSigner(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			s, err := NewSigner(tt.fi, tt.opts...)
 			if got, want := err, tt.wantErr; !errors.Is(got, want) {
@@ -540,7 +563,7 @@ func TestNewSigner(t *testing.T) {
 							got = append(got, od.ID())
 						}
 
-						if !reflect.DeepEqual(got, want) {
+						if !slices.Equal(got, want) {
 							t.Errorf("got objects %v, want %v", got, want)
 						}
 					}
@@ -550,108 +573,274 @@ func TestNewSigner(t *testing.T) {
 	}
 }
 
+//nolint:maintidx
 func TestSigner_Sign(t *testing.T) {
 	e := getTestEntity(t)
 
 	encrypted := getTestEntity(t)
-	encrypted.PrivateKey.Encrypted = true
+	if err := encrypted.PrivateKey.Encrypt([]byte("blah")); err != nil {
+		t.Fatal(err)
+	}
+
+	ss := getTestSigner(t, "ed25519-private.pem", crypto.Hash(0))
+	sv := getTestVerifier(t, "ed25519-public.pem", crypto.Hash(0))
 
 	tests := []struct {
-		name      string
-		inputFile string
-		opts      []SignerOpt
-		wantErr   bool
+		name       string
+		inputFile  string
+		signOpts   []SignerOpt
+		verifyOpts []VerifierOpt
+		wantErr    bool
 	}{
 		{
 			name:      "EncryptedKey",
 			inputFile: "one-group.sif",
-			opts:      []SignerOpt{OptSignWithEntity(encrypted)},
+			signOpts:  []SignerOpt{OptSignWithEntity(encrypted)},
 			wantErr:   true,
 		},
 		{
-			name:      "OneGroup",
+			name:      "OneGroupDSSE",
 			inputFile: "one-group.sif",
-			opts: []SignerOpt{
-				OptSignWithEntity(e),
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
 				OptSignWithTime(fixedTime),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+			},
 		},
 		{
-			name:      "TwoGroups",
-			inputFile: "two-groups.sif",
-			opts: []SignerOpt{
+			name:      "OneGroupPGP",
+			inputFile: "one-group.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+			},
 		},
 		{
-			name:      "OptSignGroup1",
+			name:      "TwoGroupsDSSE",
 			inputFile: "two-groups.sif",
-			opts: []SignerOpt{
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+			},
+		},
+		{
+			name:      "TwoGroupsPGP",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
+				OptSignWithEntity(e),
+				OptSignWithTime(fixedTime),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+			},
+		},
+		{
+			name:      "OptSignGroup1DSSE",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+				OptSignGroup(1),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+				OptVerifyGroup(1),
+			},
+		},
+		{
+			name:      "OptSignGroup1PGP",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 				OptSignGroup(1),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+				OptVerifyGroup(1),
+			},
 		},
 		{
-			name:      "OptSignGroup2",
+			name:      "OptSignGroup2DSSE",
 			inputFile: "two-groups.sif",
-			opts: []SignerOpt{
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+				OptSignGroup(2),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+				OptVerifyGroup(2),
+			},
+		},
+		{
+			name:      "OptSignGroup2PGP",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 				OptSignGroup(2),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+				OptVerifyGroup(2),
+			},
 		},
 		{
-			name:      "OptSignObject1",
+			name:      "OptSignObject1DSSE",
 			inputFile: "two-groups.sif",
-			opts: []SignerOpt{
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+				OptSignObjects(1),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+				OptVerifyObject(1),
+			},
+		},
+		{
+			name:      "OptSignObject1PGP",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 				OptSignObjects(1),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+				OptVerifyObject(1),
+			},
 		},
 		{
-			name:      "OptSignObject2",
+			name:      "OptSignObject2DSSE",
 			inputFile: "two-groups.sif",
-			opts: []SignerOpt{
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+				OptSignObjects(2),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+				OptVerifyObject(2),
+			},
+		},
+		{
+			name:      "OptSignObject2PGP",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 				OptSignObjects(2),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+				OptVerifyObject(2),
+			},
 		},
 		{
-			name:      "OptSignObject3",
+			name:      "OptSignObject3DSSE",
 			inputFile: "two-groups.sif",
-			opts: []SignerOpt{
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+				OptSignObjects(3),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+				OptVerifyObject(3),
+			},
+		},
+		{
+			name:      "OptSignObject3PGP",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 				OptSignObjects(3),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+				OptVerifyObject(3),
+			},
 		},
 		{
-			name:      "OptSignObjects",
+			name:      "OptSignObjectsDSSE",
 			inputFile: "two-groups.sif",
-			opts: []SignerOpt{
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+				OptSignObjects(1, 2, 3),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+				OptVerifyObject(1),
+				OptVerifyObject(2),
+				OptVerifyObject(3),
+			},
+		},
+		{
+			name:      "OptSignObjectsPGP",
+			inputFile: "two-groups.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 				OptSignObjects(1, 2, 3),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+				OptVerifyObject(1),
+				OptVerifyObject(2),
+				OptVerifyObject(3),
+			},
 		},
 		{
-			name:      "OptSignDeterministic",
+			name:      "OptSignDeterministicDSSE",
 			inputFile: "one-group.sif",
-			opts: []SignerOpt{
+			signOpts: []SignerOpt{
+				OptSignWithSigner(ss),
+				OptSignWithTime(fixedTime),
+				OptSignDeterministic(),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithVerifier(sv),
+			},
+		},
+		{
+			name:      "OptSignDeterministicPGP",
+			inputFile: "one-group.sif",
+			signOpts: []SignerOpt{
 				OptSignWithEntity(e),
 				OptSignWithTime(fixedTime),
 				OptSignDeterministic(),
 			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+			},
+		},
+		{
+			name:      "OptSignWithoutPGPSignatureSalt",
+			inputFile: "one-group.sif",
+			signOpts: []SignerOpt{
+				OptSignWithEntity(e),
+				OptSignWithTime(fixedTime),
+				OptSignWithoutPGPSignatureSalt(),
+			},
+			verifyOpts: []VerifierOpt{
+				OptVerifyWithKeyRing(openpgp.EntityList{e}),
+			},
 		},
 	}
 
 	for _, tt := range tests {
-		tt := tt
-
 		t.Run(tt.name, func(t *testing.T) {
 			b, err := os.ReadFile(filepath.Join(corpus, tt.inputFile))
 			if err != nil {
@@ -670,7 +859,7 @@ func TestSigner_Sign(t *testing.T) {
 				}
 			})
 
-			s, err := NewSigner(f, tt.opts...)
+			s, err := NewSigner(f, tt.signOpts...)
 			if err != nil {
 				t.Fatal(err)
 			}
@@ -679,8 +868,16 @@ func TestSigner_Sign(t *testing.T) {
 				t.Fatalf("got error %v, wantErr %v", err, tt.wantErr)
 			}
 
-			g := goldie.New(t, goldie.WithTestNameForDir(true))
-			g.Assert(t, tt.name, buf.Bytes())
+			if !tt.wantErr {
+				v, err := NewVerifier(f, tt.verifyOpts...)
+				if err != nil {
+					t.Fatal(err)
+				}
+
+				if err := v.Verify(); err != nil {
+					t.Fatal(err)
+				}
+			}
 		})
 	}
 }
