@@ -1,4 +1,4 @@
-// Copyright (c) 2020-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2020-2024, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the LICENSE.md file
 // distributed with the sources of this project regarding your rights to use or distribute this
 // software.
@@ -7,22 +7,27 @@ package integrity
 
 import (
 	"bytes"
+	"context"
 	"crypto"
 	"encoding/hex"
 	"encoding/json"
 	"errors"
 	"fmt"
 	"io"
-	"sort"
+	"slices"
 	"strings"
 
 	"github.com/ProtonMail/go-crypto/openpgp"
+	"github.com/sigstore/sigstore/pkg/signature"
 	"github.com/sylabs/sif/v2/pkg/sif"
 )
 
 var (
-	errFingerprintMismatch = errors.New("fingerprint in descriptor does not correspond to signing entity")
-	errNonGroupedObject    = errors.New("non-signature object not associated with object group")
+	errFingerprintMismatch          = errors.New("fingerprint in descriptor does not correspond to signing entity")
+	errNonGroupedObject             = errors.New("non-signature object not associated with object group")
+	errNoKeyMaterialDSSE            = errors.New("key material not provided for DSSE envelope signature")
+	errNoKeyMaterialPGP             = errors.New("key material not provided for PGP clear-sign signature")
+	errSignatureFormatNotRecognized = errors.New("signature format not recognized")
 )
 
 // SignatureNotValidError records an error when an invalid signature is encountered.
@@ -54,7 +59,6 @@ func (e *SignatureNotValidError) Unwrap() error {
 // Is compares e against target. If target is a SignatureNotValidError and matches e or target has
 // a zero value ID, true is returned.
 func (e *SignatureNotValidError) Is(target error) bool {
-	//nolint:errorlint // don't compare wrapped errors in Is()
 	t, ok := target.(*SignatureNotValidError)
 	if !ok {
 		return false
@@ -106,14 +110,14 @@ func (v *groupVerifier) signatures() ([]sif.Descriptor, error) {
 // If verification of the SIF global header fails, ErrHeaderIntegrity is returned. If verification
 // of a data object descriptor fails, a DescriptorIntegrityError is returned. If verification of a
 // data object fails, a ObjectIntegrityError is returned.
-func (v *groupVerifier) verifySignature(sig sif.Descriptor, de decoder, vr *VerifyResult) error {
+func (v *groupVerifier) verifySignature(ctx context.Context, sig sif.Descriptor, de decoder, vr *VerifyResult) error {
 	ht, fp, err := sig.SignatureMetadata()
 	if err != nil {
 		return err
 	}
 
 	// Verify signature and decode message.
-	b, err := de.verifyMessage(sig.GetReader(), ht, vr)
+	b, err := de.verifyMessage(ctx, sig.GetReader(), ht, vr)
 	if err != nil {
 		return &SignatureNotValidError{ID: sig.ID(), Err: err}
 	}
@@ -177,9 +181,9 @@ func (v *legacyGroupVerifier) signatures() ([]sif.Descriptor, error) {
 // If an invalid signature is encountered, a SignatureNotValidError is returned.
 //
 // If verification of a data object fails, a ObjectIntegrityError is returned.
-func (v *legacyGroupVerifier) verifySignature(sig sif.Descriptor, de decoder, vr *VerifyResult) error {
+func (v *legacyGroupVerifier) verifySignature(ctx context.Context, sig sif.Descriptor, de decoder, vr *VerifyResult) error { //nolint:lll
 	// Verify signature and decode message.
-	b, err := de.verifyMessage(sig.GetReader(), crypto.SHA256, vr)
+	b, err := de.verifyMessage(ctx, sig.GetReader(), crypto.SHA256, vr)
 	if err != nil {
 		return &SignatureNotValidError{ID: sig.ID(), Err: err}
 	}
@@ -242,9 +246,9 @@ func (v *legacyObjectVerifier) signatures() ([]sif.Descriptor, error) {
 // If an invalid signature is encountered, a SignatureNotValidError is returned.
 //
 // If verification of a data object fails, a ObjectIntegrityError is returned.
-func (v *legacyObjectVerifier) verifySignature(sig sif.Descriptor, de decoder, vr *VerifyResult) error {
+func (v *legacyObjectVerifier) verifySignature(ctx context.Context, sig sif.Descriptor, de decoder, vr *VerifyResult) error { //nolint:lll
 	// Verify signature and decode message.
-	b, err := de.verifyMessage(sig.GetReader(), crypto.SHA256, vr)
+	b, err := de.verifyMessage(ctx, sig.GetReader(), crypto.SHA256, vr)
 	if err != nil {
 		return &SignatureNotValidError{ID: sig.ID(), Err: err}
 	}
@@ -281,7 +285,7 @@ func (v *legacyObjectVerifier) verifySignature(sig sif.Descriptor, de decoder, v
 type decoder interface {
 	// verifyMessage reads a message from r, verifies its signature, and returns the message
 	// contents.
-	verifyMessage(r io.Reader, h crypto.Hash, vr *VerifyResult) ([]byte, error)
+	verifyMessage(ctx context.Context, r io.Reader, h crypto.Hash, vr *VerifyResult) ([]byte, error)
 }
 
 type verifyTask interface {
@@ -297,21 +301,31 @@ type verifyTask interface {
 	// If verification of the SIF global header fails, ErrHeaderIntegrity is returned. If
 	// verification of a data object descriptor fails, a DescriptorIntegrityError is returned. If
 	// verification of a data object fails, a ObjectIntegrityError is returned.
-	verifySignature(sig sif.Descriptor, de decoder, vr *VerifyResult) error
+	verifySignature(ctx context.Context, sig sif.Descriptor, de decoder, vr *VerifyResult) error
 }
 
 type verifyOpts struct {
+	vs          []signature.Verifier
 	kr          openpgp.KeyRing
 	groups      []uint32
 	objects     []uint32
 	isLegacy    bool
 	isLegacyAll bool
+	ctx         context.Context //nolint:containedctx
 	cb          VerifyCallback
 }
 
 // VerifierOpt are used to configure vo.
 type VerifierOpt func(vo *verifyOpts) error
 
+// OptVerifyWithVerifier appends verifier(s) to the sources of key material used for verification.
+func OptVerifyWithVerifier(vs ...signature.Verifier) VerifierOpt {
+	return func(vo *verifyOpts) error {
+		vo.vs = append(vo.vs, vs...)
+		return nil
+	}
+}
+
 // OptVerifyWithKeyRing sets the keyring to use for verification to kr.
 func OptVerifyWithKeyRing(kr openpgp.KeyRing) VerifierOpt {
 	return func(vo *verifyOpts) error {
@@ -372,6 +386,15 @@ func OptVerifyLegacyAll() VerifierOpt {
 	}
 }
 
+// OptVerifyWithContext specifies that the given context should be used in RPC to external
+// services.
+func OptVerifyWithContext(ctx context.Context) VerifierOpt {
+	return func(vo *verifyOpts) error {
+		vo.ctx = ctx
+		return nil
+	}
+}
+
 // OptVerifyCallback registers cb as the verification callback, which is called after each
 // signature is verified.
 func OptVerifyCallback(cb VerifyCallback) VerifierOpt {
@@ -436,17 +459,18 @@ func getLegacyTasks(f *sif.FileImage, groupIDs, objectIDs []uint32) ([]verifyTas
 // Verifier describes a SIF image verifier.
 type Verifier struct {
 	f     *sif.FileImage
+	opts  verifyOpts
 	tasks []verifyTask
-	kr    openpgp.KeyRing
-	cb    VerifyCallback
+	dsse  decoder
+	cs    decoder
 }
 
 // NewVerifier returns a Verifier to examine and/or verify digital signatures(s) in f according to
 // opts.
 //
-// Verify requires key material be provided. OptVerifyWithKeyRing can be used for this purpose. Key
-// material is not required for routines that do not perform cryptographic verification, such as
-// AnySignedBy or AllSignedBy.
+// Verify requires key material be provided. OptVerifyWithVerifier and/or OptVerifyWithKeyRing can
+// be used for this purpose. Key material is not required for routines that do not perform
+// cryptographic verification, such as AnySignedBy or AllSignedBy.
 //
 // By default, the returned Verifier will consider non-legacy signatures for all object groups. To
 // override this behavior, consider using OptVerifyGroup, OptVerifyObject, OptVerifyLegacy, and/or
@@ -456,7 +480,9 @@ func NewVerifier(f *sif.FileImage, opts ...VerifierOpt) (*Verifier, error) {
 		return nil, fmt.Errorf("integrity: %w", errNilFileImage)
 	}
 
-	vo := verifyOpts{}
+	vo := verifyOpts{
+		ctx: context.Background(),
+	}
 
 	// Apply options.
 	for _, o := range opts {
@@ -496,17 +522,25 @@ func NewVerifier(f *sif.FileImage, opts ...VerifierOpt) (*Verifier, error) {
 
 	v := Verifier{
 		f:     f,
+		opts:  vo,
 		tasks: t,
-		kr:    vo.kr,
-		cb:    vo.cb,
 	}
+
+	if vo.vs != nil {
+		v.dsse = newDSSEDecoder(vo.vs...)
+	}
+
+	if vo.kr != nil {
+		v.cs = newClearsignDecoder(vo.kr)
+	}
+
 	return &v, nil
 }
 
 // fingerprints returns a sorted list of unique fingerprints of entities participating in the
-// verification tasks in v. If any is true, entities involved in at least one task are included.
-// Otherwise, only entities participatinging in all tasks are included.
-func (v *Verifier) fingerprints(any bool) ([][]byte, error) {
+// verification tasks in v. If anyTask is true, entities involved in at least one task are
+// included. Otherwise, only entities participatinging in all tasks are included.
+func (v *Verifier) fingerprints(anyTask bool) ([][]byte, error) {
 	m := make(map[string]int)
 
 	// Build up a map containing fingerprints, and the number of tasks they are participating in.
@@ -529,7 +563,7 @@ func (v *Verifier) fingerprints(any bool) ([][]byte, error) {
 	// Build up list of fingerprints.
 	var fps [][]byte
 	for fp, n := range m {
-		if any || len(v.tasks) == n {
+		if anyTask || len(v.tasks) == n {
 			b, err := hex.DecodeString(fp)
 			if err != nil {
 				panic(err)
@@ -538,9 +572,7 @@ func (v *Verifier) fingerprints(any bool) ([][]byte, error) {
 		}
 	}
 
-	sort.Slice(fps, func(i, j int) bool {
-		return bytes.Compare(fps[i], fps[j]) < 0
-	})
+	slices.SortFunc(fps, bytes.Compare)
 
 	return fps, nil
 }
@@ -573,8 +605,7 @@ func (v *Verifier) AllSignedBy() ([][]byte, error) {
 
 // Verify performs all cryptographic verification tasks specified by v.
 //
-// If key material was not provided when v was created, Verify returns an error wrapping
-// ErrNoKeyMaterial.
+// If appropriate key material was not provided when v was created, Verify returns an error.
 //
 // If no signatures are found for a task specified by v, an error wrapping a SignatureNotFoundError
 // is returned. If an invalid signature is encountered, an error wrapping a SignatureNotValidError
@@ -585,15 +616,6 @@ func (v *Verifier) AllSignedBy() ([][]byte, error) {
 // DescriptorIntegrityError is returned. If verification of a data object fails, an error wrapping
 // a ObjectIntegrityError is returned.
 func (v *Verifier) Verify() error {
-	// Get message decoder.
-	var de decoder
-	switch {
-	case v.kr != nil:
-		de = newClearsignDecoder(v.kr)
-	default:
-		return fmt.Errorf("integrity: %w", ErrNoKeyMaterial)
-	}
-
 	// All non-signature objects must be contained in an object group.
 	ods, err := v.f.GetDescriptors(sif.WithNoGroup())
 	if err != nil {
@@ -613,15 +635,32 @@ func (v *Verifier) Verify() error {
 		}
 
 		for _, sig := range sigs {
+			// Get decoder based on signature type.
+			var de decoder
+			switch {
+			case isDSSESignature(sig.GetReader()):
+				if v.dsse == nil {
+					return fmt.Errorf("integrity: %w", errNoKeyMaterialDSSE)
+				}
+				de = v.dsse
+			case isClearsignSignature(sig.GetReader()):
+				if v.cs == nil {
+					return fmt.Errorf("integrity: %w", errNoKeyMaterialPGP)
+				}
+				de = v.cs
+			default:
+				return fmt.Errorf("integrity: %w", errSignatureFormatNotRecognized)
+			}
+
 			vr := VerifyResult{sig: sig}
 
 			// Verify signature.
-			err := t.verifySignature(sig, de, &vr)
+			err := t.verifySignature(v.opts.ctx, sig, de, &vr)
 
 			// Call verify callback, if applicable.
-			if v.cb != nil {
+			if v.opts.cb != nil {
 				vr.err = err
-				if ignoreError := v.cb(vr); ignoreError {
+				if ignoreError := v.opts.cb(vr); ignoreError {
 					err = nil
 				}
 			}
