@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2018-2024, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -8,18 +8,21 @@ package sif
 import (
 	"bytes"
 	"crypto"
+	"encoding/json"
 	"errors"
 	"io"
 	"os"
 	"path/filepath"
+	"reflect"
 	"testing"
 
+	v1 "github.com/google/go-containerregistry/pkg/v1"
 	"github.com/sebdah/goldie/v2"
 )
 
 func TestDescriptor_GetData(t *testing.T) {
 	f, err := LoadContainerFromPath(
-		filepath.Join(corpus, "one-group-signed.sif"),
+		filepath.Join(corpus, "one-group-signed-pgp.sif"),
 		OptLoadWithFlag(os.O_RDONLY),
 	)
 	if err != nil {
@@ -49,7 +52,7 @@ func TestDescriptor_GetData(t *testing.T) {
 
 func TestDescriptor_GetReader(t *testing.T) {
 	f, err := LoadContainerFromPath(
-		filepath.Join(corpus, "one-group-signed.sif"),
+		filepath.Join(corpus, "one-group-signed-pgp.sif"),
 		OptLoadWithFlag(os.O_RDONLY),
 	)
 	if err != nil {
@@ -102,6 +105,57 @@ func TestDescriptor_Name(t *testing.T) {
 	}
 }
 
+type testMetadata struct {
+	Value int
+}
+
+func (m testMetadata) MarshalBinary() ([]byte, error) {
+	return json.Marshal(m)
+}
+
+func (m *testMetadata) UnmarshalBinary(b []byte) error {
+	return json.Unmarshal(bytes.TrimRight(b, "\x00"), m)
+}
+
+func TestDescriptor_GetMetadata(t *testing.T) {
+	md := testMetadata{100}
+
+	rd := rawDescriptor{
+		DataType: DataGeneric,
+	}
+	if err := rd.setExtra(md); err != nil {
+		t.Fatal(err)
+	}
+
+	tests := []struct {
+		name    string
+		rd      rawDescriptor
+		wantMD  testMetadata
+		wantErr error
+	}{
+		{
+			name:   "OK",
+			rd:     rd,
+			wantMD: md,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			d := Descriptor{raw: tt.rd}
+
+			var md testMetadata
+
+			if got, want := d.GetMetadata(&md), tt.wantErr; !errors.Is(got, want) {
+				t.Fatalf("got error %v, want %v", got, want)
+			}
+
+			if got, want := md, tt.wantMD; !reflect.DeepEqual(got, want) {
+				t.Fatalf("got metadata %v, want %v", got, want)
+			}
+		})
+	}
+}
+
 func TestDescriptor_PartitionMetadata(t *testing.T) {
 	p := partition{
 		Fstype:   FsSquash,
@@ -208,7 +262,7 @@ func TestDescriptor_SignatureMetadata(t *testing.T) {
 			rd := rawDescriptor{
 				DataType: tt.dt,
 			}
-			if err := rd.setExtra(sig); err != nil {
+			if err := rd.setExtra(binaryMarshaler{sig}); err != nil {
 				t.Fatal(err)
 			}
 
@@ -242,7 +296,7 @@ func TestDescriptor_CryptoMessageMetadata(t *testing.T) {
 	rd := rawDescriptor{
 		DataType: DataCryptoMessage,
 	}
-	if err := rd.setExtra(m); err != nil {
+	if err := rd.setExtra(binaryMarshaler{m}); err != nil {
 		t.Fatal(err)
 	}
 
@@ -298,7 +352,7 @@ func TestDescriptor_SBOMMetadata(t *testing.T) {
 	rd := rawDescriptor{
 		DataType: DataSBOM,
 	}
-	if err := rd.setExtra(m); err != nil {
+	if err := rd.setExtra(binaryMarshaler{m}); err != nil {
 		t.Fatal(err)
 	}
 
@@ -340,6 +394,53 @@ func TestDescriptor_SBOMMetadata(t *testing.T) {
 	}
 }
 
+func TestDescriptor_OCIBlobMetadata(t *testing.T) {
+	rd := rawDescriptor{
+		DataType: DataOCIBlob,
+	}
+	if err := rd.setExtra(newOCIBlobDigest()); err != nil {
+		t.Fatal(err)
+	}
+
+	tests := []struct {
+		name       string
+		rd         rawDescriptor
+		wantDigest v1.Hash
+		wantErr    error
+	}{
+		{
+			name: "UnexpectedDataType",
+			rd: rawDescriptor{
+				DataType: DataGeneric,
+			},
+			wantErr: &unexpectedDataTypeError{DataGeneric, []DataType{DataOCIBlob, DataOCIRootIndex}},
+		},
+		{
+			name: "OK",
+			rd:   rd,
+			wantDigest: v1.Hash{
+				Algorithm: "sha256",
+				Hex:       "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855",
+			},
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			d := Descriptor{raw: tt.rd}
+
+			h, err := d.OCIBlobDigest()
+
+			if got, want := err, tt.wantErr; !errors.Is(got, want) {
+				t.Fatalf("got error %v, want %v", got, want)
+			}
+
+			if got, want := h, tt.wantDigest; got != want {
+				t.Fatalf("got digest %v, want %v", got, want)
+			}
+		})
+	}
+}
+
 func TestDescriptor_GetIntegrityReader(t *testing.T) {
 	rd := rawDescriptor{
 		DataType:   DataDeffile,
@@ -420,7 +521,6 @@ func TestDescriptor_GetIntegrityReader(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
 			d := Descriptor{
 				raw:        rd,
