@@ -1,4 +1,4 @@
-// Copyright (c) 2021-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2021-2023, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -33,6 +33,18 @@ func Test_command_getAdd(t *testing.T) {
 				"--signentity", "433FE984155206BD962725E20E8713472A879943",
 			},
 		},
+		{
+			name: "DataOCIRootIndex",
+			flags: []string{
+				"--datatype", "10",
+			},
+		},
+		{
+			name: "DataOCIBlob",
+			flags: []string{
+				"--datatype", "11",
+			},
+		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
