@@ -1,4 +1,4 @@
-// Copyright (c) 2021-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2021-2025, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -6,7 +6,7 @@
 package siftool
 
 import (
-	"os"
+	"path/filepath"
 	"testing"
 )
 
@@ -21,18 +21,13 @@ func Test_command_getNew(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			tf, err := os.CreateTemp("", "sif-test-*")
-			if err != nil {
-				t.Fatal(err)
-			}
-			defer os.Remove(tf.Name())
-			tf.Close()
+			path := filepath.Join(t.TempDir(), "sif")
 
 			c := &command{opts: tt.opts}
 
 			cmd := c.getNew()
 
-			runCommand(t, cmd, []string{tf.Name()}, nil)
+			runCommand(t, cmd, []string{path}, nil)
 		})
 	}
 }
