@@ -1,4 +1,4 @@
-// Copyright (c) 2021-2022, Sylabs Inc. All rights reserved.
+// Copyright (c) 2021-2025, Sylabs Inc. All rights reserved.
 // This software is licensed under a 3-clause BSD license. Please consult the
 // LICENSE file distributed with the sources of this project regarding your
 // rights to use or distribute this software.
@@ -7,7 +7,6 @@ package siftool
 import (
 	"bytes"
 	"errors"
-	"os"
 	"path/filepath"
 	"testing"
 
@@ -19,25 +18,21 @@ import (
 
 var corpus = filepath.Join("..", "..", "test", "images")
 
+//nolint:thelper // Complex enough to justify keeping file/line information on error.
 func makeTestSIF(t *testing.T, withDataObject bool) string {
-	tf, err := os.CreateTemp("", "sif-test-*")
-	if err != nil {
-		t.Fatal(err)
-	}
-	t.Cleanup(func() { os.Remove(tf.Name()) })
-	tf.Close()
-
 	app, err := siftool.New()
 	if err != nil {
 		t.Fatal(err)
 	}
 
-	if err := app.New(tf.Name()); err != nil {
+	path := filepath.Join(t.TempDir(), "sif")
+
+	if err := app.New(path); err != nil {
 		t.Fatal(err)
 	}
 
 	if withDataObject {
-		err := app.Add(tf.Name(), sif.DataPartition, bytes.NewReader([]byte{0xde, 0xad, 0xbe, 0xef}),
+		err := app.Add(path, sif.DataPartition, bytes.NewReader([]byte{0xde, 0xad, 0xbe, 0xef}),
 			sif.OptPartitionMetadata(sif.FsSquash, sif.PartSystem, "386"),
 		)
 		if err != nil {
@@ -45,9 +40,10 @@ func makeTestSIF(t *testing.T, withDataObject bool) string {
 		}
 	}
 
-	return tf.Name()
+	return path
 }
 
+//nolint:unparam
 func runCommand(t *testing.T, cmd *cobra.Command, args []string, wantErr error) {
 	t.Helper()
 
@@ -116,11 +112,6 @@ func TestAddCommands(t *testing.T) {
 			name: "SetPrim",
 			args: []string{"help", "setprim"},
 		},
-		{
-			name: "Mount",
-			opts: []CommandOpt{OptWithExperimental(true)},
-			args: []string{"help", "mount"},
-		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
